import click

from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    TRANSACTIONAL_WAREHOUSE_TYPE,
    TRANSACTIONAL_CONNECTION_TYPE,
)


class TransactionalOnboardingService(CollectorValidationService, BaseOnboardingService):
    def __init__(self, config: Config, **kwargs):
        super().__init__(config, **kwargs)

    @manage_errors
    def onboard_transactional_db(self, **kwargs) -> None:
        """
        Onboard a Transactional DB connection by validating and adding a connection.
        """
        kwargs["connection_type"] = TRANSACTIONAL_CONNECTION_TYPE
        kwargs["warehouse_type"] = TRANSACTIONAL_WAREHOUSE_TYPE

        # Finds all v2 validations for this connection and runs each one.
        # If they all pass, return a temp credentials key
        # If the --skip-validations flag has been used, no validations are run and just a temp key is returned
        key = self.test_new_credentials(**kwargs)

        # Add the connection
        if key:
            self.add_connection(key, **kwargs)
        else:
            click.Abort()



import copy
import json
import re
import uuid
from typing import Optional, List, Dict, Any

import click
from montecarlodata.errors import prompt_connection
from pycarlo.common.errors import GqlError
from pycarlo.core import Client, Query
from pycarlo.lib.schema import TestCredentialsV2Response

from montecarlodata.common.echo_utils import (
    echo_warning_message,
    echo_error_message,
    styled_warning_icon,
    styled_error_icon,
    styled_success_icon,
    echo_success_message,
)
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort
from montecarlodata.utils import GqlWrapper

from montecarlodata.integrations.onboarding.fields import (
    CONFLUENT_KAFKA_CONNECTION_TYPE,
    CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE,
    EXPECTED_TEST_PINECONE_CREDENTIALS_RESPONSE_FIELD,
    EXPECTED_TEST_CONFLUENT_KAFKA_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_CONFLUENT_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_SELF_HOSTED_KAFKA_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
    SELF_HOSTED_KAFKA_CONNECTION_TYPE,
    SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
    PINECONE_CONNECTION_TYPE,
    TRANSACTIONAL_CONNECTION_TYPE,
    EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    TEST_CONFLUENT_KAFKA_CRED_MUTATION,
    TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
    TEST_PINECONE_CRED_MUTATION,
    TEST_TRANSACTIONAL_DB_CRED_MUTATION,
)


class CollectorValidationService:
    _SAVE_CREDENTIALS = "save_credentials"

    _CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CRED_MUTATION,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
        PINECONE_CONNECTION_TYPE: TEST_PINECONE_CRED_MUTATION,
        TRANSACTIONAL_CONNECTION_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
    }

    _CONNECTION_TYPES_TO_OPERATION_TYPE = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: EXPECTED_TEST_CONFLUENT_KAFKA_CRED_RESPONSE_FIELD,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: EXPECTED_TEST_CONFLUENT_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: EXPECTED_TEST_SELF_HOSTED_KAFKA_CRED_RESPONSE_FIELD,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: EXPECTED_TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
        PINECONE_CONNECTION_TYPE: EXPECTED_TEST_PINECONE_CREDENTIALS_RESPONSE_FIELD,
        TRANSACTIONAL_CONNECTION_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
    }

    def __init__(
        self,
        config: Config,
        mc_client: Client,
        request_wrapper: Optional[GqlWrapper] = None,
        user_service: Optional[UserService] = None,
    ):
        self._mc_client = mc_client
        self._request_wrapper = request_wrapper or GqlWrapper(config)
        self._user_service = user_service or UserService(
            request_wrapper=self._request_wrapper, config=config
        )

    def run_validations(
        self, dc_id: Optional[str] = None, only_periodic: bool = False
    ) -> int:
        """
        Runs all validators for all integrations in the data collector.

        :param dc_id: The UUID of the data collector.
        :param only_periodic: Whether only periodic validations must be run or not.
        """
        dc = self._user_service.get_collector(dc_id)

        # Filter integration list to just integrations for this dc
        all_integrations = [
            integration
            for integration in (
                (self._user_service.warehouses or [])
                + (self._user_service.bi_containers or [])
                + (self._user_service.etl_containers or [])
            )
            if integration.get("dataCollector", {}).get("uuid") == dc.uuid
        ]

        total_failures = 0
        for integration in all_integrations:
            click.echo("")
            connections: List[Dict[str, Any]] = integration.get("connections", [])
            for connection in connections:
                if "type" in connection and "uuid" in connection:
                    connection_failures = self._run_connection_validators(
                        dc_uuid=dc.uuid,
                        name=integration.get("name", ""),
                        connection_type=connection["type"].lower(),
                        connection_uuid=connection["uuid"],
                        only_periodic=only_periodic,
                    )
                    total_failures += connection_failures

        # Run storage access validation. If DC has an agent registered, will validate that the agent's storage location is accessible.
        # If DC has no agent registered, will validate that default storage location is accessible.
        click.echo("")
        total_failures += self._run_storage_access_validation(dc_id=dc.uuid)

        click.echo("")
        if total_failures == 0:
            click.echo("All validations passed")
        else:
            click.echo(f"{total_failures} validations failed")
        return total_failures

    def _run_connection_validators(
        self,
        dc_uuid: uuid.UUID,
        name: str,
        connection_type: str,
        connection_uuid: uuid.UUID,
        only_periodic: bool,
    ) -> int:
        click.echo(
            f"Validating {connection_type} connection: {name} - {connection_uuid}"
        )
        validations = self._get_supported_validations(
            dc_uuid=dc_uuid,
            connection_type=connection_type,
            only_periodic=only_periodic,
        )
        failures = 0
        for validation in validations:
            ok = False
            warning = False
            try:
                result = self._run_single_validation(
                    connection_uuid=connection_uuid,
                    validation_name=validation,
                )

                ok, warning = self._process_validation_result(result)
            except Exception as e:
                echo_error_message(f"\tValidation {validation} failed ({e}).")

            result_icon, failures = self._get_icon(ok, warning, failures)
            click.echo(f"\t{validation}: {result_icon}")

        return failures

    def _get_supported_validations(
        self, dc_uuid: uuid.UUID, connection_type: str, only_periodic: bool
    ) -> List[str]:
        query = Query()
        query.get_supported_validations_v2(
            dc_id=dc_uuid, connection_type=connection_type
        )

        try:
            result = self._mc_client(
                query=query,
            ).get_supported_validations_v2

            return [
                validation.name
                for validation in result.supported_validations
                if not only_periodic or validation.periodic_validation
            ]
        except GqlError as e:
            complain_and_abort(f"Unable to get list of supported validators ({e}).")
            return []

    def _run_single_validation(
        self,
        connection_uuid: uuid.UUID,
        validation_name: str,
    ) -> Optional[TestCredentialsV2Response]:
        query = Query()
        query.test_existing_connection_v2(
            connection_id=connection_uuid, validation_name=validation_name
        )

        return self._mc_client(
            query=query,
            idempotent_request_id=str(uuid.uuid4()),
            timeout_in_seconds=40,  # let Monolith timeout first
        ).test_existing_connection_v2

    def _run_storage_access_validation(
        self,
        dc_id: uuid.UUID,
    ) -> int:
        query = Query()
        query.test_storage_access(dc_id=dc_id)

        click.echo(f"Validating storage access:")

        failures = 0
        ok = False
        warning = False
        try:
            result = self._mc_client(
                query=query,
                idempotent_request_id=str(uuid.uuid4()),
                timeout_in_seconds=40,  # let Monolith timeout first
            ).test_storage_access

            ok, warning = self._process_validation_result(result)
        except Exception as e:
            echo_error_message(f"\tValidation validate_storage_access failed ({e}).")

        result_icon, failures = self._get_icon(ok, warning, failures)
        click.echo(f"\tvalidate_storage_access: {result_icon}")

        return failures

    @staticmethod
    def _process_validation_result(result: TestCredentialsV2Response) -> (bool, bool):
        ok = False
        warning = False

        # if there are only warnings we log them but consider it a successful validation
        ok = result.success or not bool(result.errors)
        if result.warnings:
            for warning in result.warnings:
                echo_warning_message(f"\t{warning.cause}")
        if result.errors:
            for error in result.errors:
                echo_error_message(f"\t{error.cause}")
        elif not result.success:
            warning = True

        return ok, warning

    @staticmethod
    def _get_icon(ok: bool, warning: bool, failures: int) -> (str, int):
        if ok:
            result_icon = styled_warning_icon() if warning else styled_success_icon()
        else:
            result_icon = styled_error_icon()
            failures += 1
        return result_icon, failures

    def test_new_credentials(
        self,
        connection_type: str,
        dc_id: Optional[uuid.UUID] = None,
        generate_key: Optional[bool] = True,
        skip_validation: Optional[bool] = False,
        validate_only: Optional[bool] = False,
        **kwargs,
    ) -> Optional[str]:
        if not dc_id:
            dc_id = self._user_service.get_collector().uuid

        kwargs["dc_id"] = str(dc_id)

        if not skip_validation:
            supported_validation_tests = self._get_supported_validations(
                dc_uuid=str(dc_id),
                connection_type=connection_type,
                only_periodic=False,
            )
            for validation_name in supported_validation_tests:
                variables = copy.deepcopy(kwargs)
                variables["validation_name"] = validation_name

                operation_name = self._CONNECTION_TYPES_TO_OPERATION_TYPE[
                    connection_type
                ]
                operation_name_snake = self._camel_to_snake(operation_name)
                validation_result = (
                    self._mc_client(
                        query=self._CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING[
                            connection_type
                        ],
                        operation_name=operation_name,
                        variables=GqlWrapper.convert_snakes_to_camels(variables),
                        idempotent_request_id=str(uuid.uuid4()),
                        timeout_in_seconds=40,  # let Monolith timeout first
                    )
                    .__getattr__(operation_name_snake)
                    .validation_result
                )
                ok, warning = self._process_validation_result(result=validation_result)

                result_icon, _ = self._get_icon(ok, warning, 0)
                click.echo(f"\t{validation_name}: {result_icon}")

                if not ok:
                    # validation failed
                    raise click.Abort()

        if not validate_only:
            prompt_connection(
                message="Validations passed! Would you like to continue?",
                skip_prompt=not generate_key,
            )
            variables = copy.deepcopy(kwargs)
            variables["validation_name"] = self._SAVE_CREDENTIALS
            new_key_response = self._request_wrapper.make_request_v2(
                query=self._CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING[
                    connection_type
                ],
                operation=self._CONNECTION_TYPES_TO_OPERATION_TYPE[connection_type],
                variables=variables,
            )

            validation_result = new_key_response.data.validationResult
            if not validation_result.success:
                click.echo("creating credential key failed.")
                click.echo(json.dumps(validation_result))
                click.Abort()

            return new_key_response.data.key
        else:
            echo_success_message(message="Validations passed!")

    @staticmethod
    def _camel_to_snake(text: str) -> str:
        return re.sub("([A-Z]+)", r"_\1", text).lower()



from box import Box
import click
from copy import deepcopy
from typing import Dict, Optional
import uuid
from pycarlo.core import Client

from montecarlodata.errors import echo_error
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    CONFLUENT_KAFKA_CLUSTER_TYPE,
    CONFLUENT_KAFKA_CONNECT_CLUSTER_TYPE,
    CONFLUENT_KAFKA_CONNECTION_TYPE,
    CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE,
    EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
    EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
    EXPECTED_GET_STREAMING_SYSTEMS_RESPONSE_FIELD,
    SELF_HOSTED_KAFKA_CLUSTER_TYPE,
    SELF_HOSTED_KAFKA_CONNECT_CLUSTER_TYPE,
    SELF_HOSTED_KAFKA_CONNECTION_TYPE,
    SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    ADD_STREAMING_SYSTEM_MUTATION,
    ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
    GET_STREAMING_SYSTEMS_QUERY,
    TEST_CONFLUENT_KAFKA_CRED_MUTATION,
    TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
)


class StreamingOnboardingService(BaseOnboardingService):
    _CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CRED_MUTATION,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
    }

    _CONNECTION_TYPES_TO_CLUSTER_TYPE_MAPPING = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: CONFLUENT_KAFKA_CLUSTER_TYPE,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: CONFLUENT_KAFKA_CONNECT_CLUSTER_TYPE,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: SELF_HOSTED_KAFKA_CLUSTER_TYPE,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: SELF_HOSTED_KAFKA_CONNECT_CLUSTER_TYPE,
    }

    def __init__(self, config: Config, mc_client: Client, **kwargs):
        super().__init__(config, **kwargs)
        self._validation_service = CollectorValidationService(
            config=config,
            mc_client=mc_client,
            user_service=self._user_service,
            request_wrapper=self._request_wrapper,
        )

    @manage_errors
    def create_streaming_system(
        self,
        streaming_system_type: str,
        streaming_system_name: str,
        dc_id: Optional[str] = None,
    ) -> None:
        """
        Creates a streaming system.
        """
        response = self._request_wrapper.make_request_v2(
            query=ADD_STREAMING_SYSTEM_MUTATION,
            operation=EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
            variables=dict(
                streaming_system_type=streaming_system_type,
                streaming_system_name=streaming_system_name,
                dc_id=dc_id,
            ),
        )
        streaming_system = response.data.streamingSystem
        click.echo(
            f"Successfully created the {streaming_system.type} streaming system "
            f"{streaming_system.name} with uuid: {streaming_system.uuid}."
        )

    def get_streaming_system(self, streaming_system_uuid: str) -> Box:
        """
        Gets a streaming system.
        """
        response = self._request_wrapper.make_request_v2(
            query=GET_STREAMING_SYSTEMS_QUERY,
            operation=EXPECTED_GET_STREAMING_SYSTEMS_RESPONSE_FIELD,
            variables=dict(
                include_clusters=False,
            ),
        )
        for entry in response.data:
            system = entry.system
            if system.uuid == streaming_system_uuid:
                return system

        echo_error(f"cannot find the stream system with uuid {streaming_system_uuid}")

    @manage_errors
    def test_new_confluent_kafka_credentials(
        self,
        cluster: str,
        api_key: str,
        secret: str,
        url: str,
        dc_id: Optional[uuid.UUID] = None,
    ) -> str:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=CONFLUENT_KAFKA_CONNECTION_TYPE,
            cluster=cluster,
            api_key=api_key,
            secret=secret,
            url=url,
        )

    @manage_errors
    def test_new_confluent_kafka_connect_credentials(
        self,
        confluent_env: str,
        cluster: str,
        api_key: str,
        secret: str,
        url: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> str:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE,
            confluent_env=confluent_env,
            cluster=cluster,
            api_key=api_key,
            secret=secret,
            url=url,
        )

    @manage_errors
    def test_new_self_hosted_kafka_credentials(
        self,
        cluster: str,
        url: str,
        auth_type: str,
        auth_token: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> str:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=SELF_HOSTED_KAFKA_CONNECTION_TYPE,
            cluster=cluster,
            auth_type=auth_type,
            auth_token=auth_token,
            url=url,
        )

    @manage_errors
    def test_new_self_hosted_kafka_connect_credentials(
        self,
        cluster: str,
        url: str,
        auth_type: str,
        auth_token: Optional[str] = None,
        dc_id: Optional[uuid.UUID] = None,
    ) -> str:
        return self._validation_service.test_new_credentials(
            dc_id=dc_id,
            connection_type=SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
            cluster=cluster,
            auth_type=auth_type,
            auth_token=auth_token,
            url=url,
        )

    @manage_errors
    def onboard_confluent_cluster_connection(self, **kwargs) -> None:
        connection_type = kwargs.get("connection_type")
        if (
            connection_type
            not in self._CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING.keys()
        ):
            echo_error("Not supported streaming connection type")
            click.Abort()

        streaming_system_id = kwargs.get("streaming_system_id")
        streaming_system = None
        if streaming_system_id:
            # if the streaming system exist, then we'll proceed, otherwise, we'd stop.
            # that's because either the streaming system doesn't exist or it doesn't belong to the user's account.
            streaming_system = self.get_streaming_system(
                streaming_system_uuid=streaming_system_id
            )

        credential_key = kwargs.get("key")
        if not credential_key:
            # copy params for creating credential key
            params_copy = deepcopy(kwargs)
            if not params_copy.get("dc_id") and params_copy.get("streaming_system_id"):
                params_copy["dc_id"] = streaming_system.dcId
            credential_key = self._test_and_generate_credential_key(
                connection_type=connection_type, params=params_copy
            )
            if credential_key:
                kwargs["key"] = credential_key
            else:
                # credentials testing fail or user doesn't want to continue
                click.Abort()

        if not kwargs.get("mc_cluster_id"):
            # for new cluster, we'll add the cluster type here.
            kwargs["new_cluster_type"] = self._CONNECTION_TYPES_TO_CLUSTER_TYPE_MAPPING[
                connection_type
            ]

        response = self._request_wrapper.make_request_v2(
            query=ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
            variables=kwargs,
        )

        connection = response.data.connection
        click.echo(
            f"Successfully created the {connection.type} connection {connection.uuid} for cluster "
            f"{connection.streamingCluster.uuid}, in streaming system "
            f"{connection.streamingCluster.streamingSystemUuid}."
        )

    def _test_and_generate_credential_key(
        self, connection_type: str, params: Dict
    ) -> str:
        parameters = {
            "cluster": params.get("new_cluster_id"),
        }
        if connection_type == CONFLUENT_KAFKA_CONNECTION_TYPE:
            parameters["url"] = params.get("url")
            parameters["api_key"] = params.get("api_key")
            parameters["secret"] = params.get("secret")
        elif connection_type == CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE:
            parameters["confluent_env"] = params.get("confluent_env")
            parameters["api_key"] = params.get("api_key")
            parameters["secret"] = params.get("secret")
            if "url" in params:
                parameters["url"] = params.get("url")
        elif (
            connection_type == SELF_HOSTED_KAFKA_CONNECTION_TYPE
            or connection_type == SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE
        ):
            parameters["url"] = params.get("url")
            parameters["auth_type"] = params.get("auth_type")
            parameters["auth_token"] = params.get("auth_token")
        else:
            click.echo(f"Not supported connection type {connection_type}")

        return self._validation_service.test_new_credentials(
            connection_type=connection_type, dc_id=params.get("dc_id"), **parameters
        )



from box import Box
import uuid
from unittest import TestCase
from unittest.mock import Mock

from pycarlo.core import Client
from montecarlodata.common.data import MonolithResponse
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.streaming.streamings import (
    StreamingOnboardingService,
)
from montecarlodata.collector.validation import CollectorValidationService

from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
    EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    ADD_STREAMING_SYSTEM_MUTATION,
    ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
)
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class StreamingOnboardingTest(TestCase):
    _EXPECTED_NEW_CONNECTION_RESPONSE = {
        "connection": {
            "id": "9511",
            "type": "CONFLUENT_KAFKA_CONNECT",
            "uuid": "de7b4ebb-1574-45c1-9433-07fae8b3fc9f",
            "streamingCluster": {
                "uuid": "c5c9f51d-06d6-47f6-90c1-27fda4160dda",
                "clusterType": "CONFLUENT_KAFKA_CONNECT",
                "externalClusterId": "test_cluster_id",
                "streamingSystemUuid": "6f5d117f-e644-41b4-aa60-f593c0f0dfd1",
            },
        }
    }

    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)
        self._mc_client_mock = Mock(autospec=Client)
        self._validation_service_mock = Mock(autospec=CollectorValidationService)

        self._service = StreamingOnboardingService(
            _SAMPLE_CONFIG,
            mc_client=self._mc_client_mock,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )
        self._service._validation_service = self._validation_service_mock

    def test_create_streaming_system(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box(
                {
                    "streamingSystem": {
                        "id": "1",
                        "createdTime": "2023-11-14T23:41:05.465076+00:00",
                        "updatedTime": "2023-11-14T23:41:05.465093+00:00",
                        "name": "test system 2",
                        "type": "CONFLUENT_CLOUD",
                        "accountUuid": "a5cbd8cc-8e91-4a41-aca4-4bf5bd320578",
                        "dcId": "c16e3efc-0d54-4088-9514-372a38506e62",
                        "deletedAt": None,
                        "uuid": "6f5d117f-e644-41b4-aa60-f593c0f0dfd1",
                    }
                }
            ),
            errors=[],
        )

        system_type = "confluent-cloud"
        system_name = "test confluent system"
        dc_id = str(uuid.uuid4())
        self._service.create_streaming_system(system_type, system_name, dc_id)

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=ADD_STREAMING_SYSTEM_MUTATION,
            operation=EXPECTED_ADD_STREAMING_SYSTEM_RESPONSE_FIELD,
            variables=dict(
                streaming_system_type=system_type,
                streaming_system_name=system_name,
                dc_id=dc_id,
            ),
        )

    def test_onboard_confluent_cluster_connection_with_key(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box(self._EXPECTED_NEW_CONNECTION_RESPONSE), errors=[]
        )

        self._service.onboard_confluent_cluster_connection(
            connection_type="confluent-kafka-connect",
            key="tmp/mykey",
            dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
            new_streaming_system_name="test",
            new_streaming_system_type="confluent-cloud",
            new_cluster_id="test_cluster_id",
            new_cluster_name="test_cluster_name",
        )

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
            variables=dict(
                connection_type="confluent-kafka-connect",
                key="tmp/mykey",
                dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
                new_streaming_system_name="test",
                new_streaming_system_type="confluent-cloud",
                new_cluster_id="test_cluster_id",
                new_cluster_name="test_cluster_name",
                new_cluster_type="confluent-kafka-connect",
            ),
        )
        self._validation_service_mock.test_new_credentials.assert_not_called()

    def test_onboard_confluent_cluster_connection_without_key(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box(self._EXPECTED_NEW_CONNECTION_RESPONSE), errors=[]
        )
        self._validation_service_mock.test_new_credentials.return_value = "tmp/mykey"

        self._service.onboard_confluent_cluster_connection(
            connection_type="confluent-kafka-connect",
            api_key="api_key",
            secret="secret",
            url="http://confluent.com",
            dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
            new_streaming_system_name="test",
            new_streaming_system_type="confluent-cloud",
            new_cluster_id="test_cluster_id",
            new_cluster_name="test_cluster_name",
        )

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=ADD_STREAMING_CLUSTER_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONFLUENT_CLUSTER_CONNECTION_RESPONSE_FIELD,
            variables=dict(
                connection_type="confluent-kafka-connect",
                key="tmp/mykey",
                dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
                new_streaming_system_name="test",
                new_streaming_system_type="confluent-cloud",
                new_cluster_id="test_cluster_id",
                new_cluster_name="test_cluster_name",
                new_cluster_type="confluent-kafka-connect",
                # following 3 only carried but not used in the call at all.
                api_key="api_key",
                secret="secret",
                url="http://confluent.com",
            ),
        )
        self._validation_service_mock.test_new_credentials.assert_called_once_with(
            connection_type="confluent-kafka-connect",
            dc_id="c16e3efc-0d54-4088-9514-372a38506e62",
            cluster="test_cluster_id",
            api_key="api_key",
            secret="secret",
            confluent_env=None,
            url="http://confluent.com",
        )
